@@ -1,27 +1,17 @@
 /*
- * "$Id: testmxml.c 381 2009-02-05 06:06:11Z mike $"
+ * "$Id: testmxml.c 451 2014-01-04 21:50:06Z msweet $"
  *
  * Test program for Mini-XML, a small XML-like file parsing library.
  *
- * Copyright 2003-2007 by Michael Sweet.
+ * Copyright 2003-2014 by Michael R Sweet.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2, or (at your option) any later version.
+ * These coded instructions, statements, and computer programs are the
+ * property of Michael R Sweet and are protected by Federal copyright
+ * law.  Distribution and use rights are outlined in the file "COPYING"
+ * which should have been included with this file.  If this file is
+ * missing or damaged, see the license at:
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * Contents:
- *
- *   main()          - Main entry for test program.
- *   sax_cb()        - SAX callback.
- *   type_cb()       - XML data type callback for mxmlLoadFile()...
- *   whitespace_cb() - Let the mxmlSaveFile() function know when to insert
- *                     newlines and tabs...
+ *     http://www.msweet.org/projects.php/Mini-XML
  */
 
 /*
@@ -132,6 +122,8 @@ main(int  argc,				/* I - Number of command-line args */
                  MXML_REAL_CALLBACK);
   mxmlLoadString(tree, "<group>opaque opaque opaque</group>",
                  MXML_OPAQUE_CALLBACK);
+  mxmlLoadString(tree, "<foo><bar><one><two>value<two>value2</two></two></one>"
+                       "</bar></foo>", MXML_OPAQUE_CALLBACK);
 
   node = tree->child;
 
@@ -260,6 +252,52 @@ main(int  argc,				/* I - Number of command-line args */
     }
   }
 
+ /*
+  * Test mxmlFindPath...
+  */
+
+  node = mxmlFindPath(tree, "*/two");
+  if (!node)
+  {
+    fputs("ERROR: Unable to find value for \"*/two\".\n", stderr);
+    mxmlDelete(tree);
+    return (1);
+  }
+  else if (node->type != MXML_OPAQUE || strcmp(node->value.opaque, "value"))
+  {
+    fputs("ERROR: Bad value for \"*/two\".\n", stderr);
+    mxmlDelete(tree);
+    return (1);
+  }
+
+  node = mxmlFindPath(tree, "foo/*/two");
+  if (!node)
+  {
+    fputs("ERROR: Unable to find value for \"foo/*/two\".\n", stderr);
+    mxmlDelete(tree);
+    return (1);
+  }
+  else if (node->type != MXML_OPAQUE || strcmp(node->value.opaque, "value"))
+  {
+    fputs("ERROR: Bad value for \"foo/*/two\".\n", stderr);
+    mxmlDelete(tree);
+    return (1);
+  }
+
+  node = mxmlFindPath(tree, "foo/bar/one/two");
+  if (!node)
+  {
+    fputs("ERROR: Unable to find value for \"foo/bar/one/two\".\n", stderr);
+    mxmlDelete(tree);
+    return (1);
+  }
+  else if (node->type != MXML_OPAQUE || strcmp(node->value.opaque, "value"))
+  {
+    fputs("ERROR: Bad value for \"foo/bar/one/two\".\n", stderr);
+    mxmlDelete(tree);
+    return (1);
+  }
+
  /*
   * Test indices...
   */
@@ -272,10 +310,10 @@ main(int  argc,				/* I - Number of command-line args */
     return (1);
   }
 
-  if (ind->num_nodes != 5)
+  if (ind->num_nodes != 10)
   {
     fprintf(stderr, "ERROR: Index of all nodes contains %d "
-                    "nodes; expected 5!\n", ind->num_nodes);
+                    "nodes; expected 10!\n", ind->num_nodes);
     mxmlIndexDelete(ind);
     mxmlDelete(tree);
     return (1);
@@ -380,7 +418,7 @@ main(int  argc,				/* I - Number of command-line args */
   * Check the mxmlDelete() works properly...
   */
 
-  for (i = 0; i < 8; i ++)
+  for (i = 0; i < 9; i ++)
   {
     if (tree->child)
       mxmlDelete(tree->child);
@@ -569,9 +607,9 @@ main(int  argc,				/* I - Number of command-line args */
       return (1);
     }
 
-    if (event_counts[MXML_SAX_DATA] != 61)
+    if (event_counts[MXML_SAX_DATA] != 60)
     {
-      fprintf(stderr, "MXML_SAX_DATA seen %d times, expected 61 times!\n",
+      fprintf(stderr, "MXML_SAX_DATA seen %d times, expected 60 times!\n",
               event_counts[MXML_SAX_DATA]);
       return (1);
     }
@@ -744,5 +782,5 @@ whitespace_cb(mxml_node_t *node,	/* I - Element node */
 
 
 /*
- * End of "$Id: testmxml.c 381 2009-02-05 06:06:11Z mike $".
+ * End of "$Id: testmxml.c 451 2014-01-04 21:50:06Z msweet $".
  */
